@@ -2,7 +2,7 @@
 # Purpose : Generic Cache Factory with various policy factories.
 # Author  : Sam Graham
 # Created : 23 Jun 2008
-# CVS     : $Id: CacheFactory.pm,v 1.24 2010-02-09 12:31:32 illusori Exp $
+# CVS     : $Id: CacheFactory.pm,v 1.3 2008-06-27 11:58:10 illusori Exp $
 ###############################################################################
 
 package Cache::CacheFactory;
@@ -20,16 +20,14 @@ use Cache::CacheFactory::Object;
 
 use base qw/Cache::Cache/;
 
-$Cache::CacheFactory::VERSION = '1.09_01';
-
-$Cache::CacheFactory::NO_MAX_SIZE = -1;
+$Cache::CacheFactory::VERSION =
+    sprintf"%d.%03d", q$Revision: 1.3 $ =~ /: (\d+)\.(\d+)/;
 
 @Cache::CacheFactory::EXPORT    = qw();
 @Cache::CacheFactory::EXPORT_OK = qw(
     best_available_storage_policy
     best_available_pruning_policy
     best_available_validity_policy
-    $NO_MAX_SIZE
     );
 %Cache::CacheFactory::EXPORT_TAGS = (
     best_available => [ qw(
@@ -90,10 +88,6 @@ sub new
     $self->{ nonwarning_missing_policies } = 1
         if $options{ nonwarning_missing_policies };
 
-    #  Do we deeply clone our data when setting it?
-    $self->{ no_deep_clone } = 1
-        if $options{ no_deep_clone };
-
     #
     #  Grab our policies from the options.
     $self->set_storage_policies(  $options{ storage  } );
@@ -126,33 +120,21 @@ sub new_cache_entry_object
 sub set
 {
     my $self = shift;
-    my ( $param, $object, $key, $data, $mode );
+    my ( $param, $object, $key, $data );
 
     #  Aiii, backwards-compat with Cache::Cache->set().
     if( $self->{ compat }->{ positional_set } and
         ( ( $self->{ compat }->{ positional_set } ne 'auto' ) or
           ( $_[ 0 ] ne 'key' ) ) )
     {
-        my ( $next_arg, $expires_in );
+        my ( $expires_in );
 
         $key        = shift;
         $data       = shift;
         $expires_in = shift;
         $param = {};
-        if( defined( $next_arg = shift ) )
-        {
-            #  Hackery to support mode from add()/replace().
-            if( $next_arg eq 'mode' )
-            {
-                $mode = shift;
-            }
-            else
-            {
-                $param->{ expires_in } = $expires_in;
-                #  TODO: warn if expires set and not time policy?
-            }
-        }
-        $mode = shift if defined( $next_arg = shift ) and $next_arg eq 'mode';
+        $param->{ expires_in } = $expires_in if defined( $expires_in = shift );
+        #  TODO: warn if expires set and not time pruning/validity policy?
     }
     else
     {
@@ -162,46 +144,14 @@ sub set
             $key  = $param->{ key };
             delete $param->{ key };
         }
-        else
-        {
-            warn "No key supplied to ${self}->set(), are you calling it " .
-                "with compat-style positional parameters but haven't set " .
-                "the positional_set option?";
-            return;
-        }
         if( exists( $param->{ data } ) )
         {
             $data = $param->{ data };
             delete $param->{ data };
         }
-        else
-        {
-            warn "No data supplied to ${self}->set(), are you calling it " .
-                "with compat-style positional parameters but haven't set " .
-                "the positional_set option?";
-            return;
-        }
-        if( exists( $param->{ mode } ) )
-        {
-            $mode = $param->{ mode };
-            delete $param->{ mode };
-        }
     }
 
-    if( $mode )
-    {
-        if( $self->exists( $key ) )
-        {
-            return if $mode eq 'add';
-        }
-        else
-        {
-            return if $mode eq 'replace';
-        }
-    }
-
-    $param->{ created_at }    = time() unless $param->{ created_at };
-    $param->{ no_deep_clone } = 1      if $self->{ no_deep_clone };
+    $param->{ created_at } = time() unless $param->{ created_at };
 
     #  Create Cache::CacheFactory::Object instance.
     $object = $self->new_cache_entry_object();
@@ -213,41 +163,8 @@ sub set
         $key, $object, $param );
     $self->foreach_driver( 'pruning',  'set_object_pruning',
         $key, $object, $param );
-    if( $param->{ no_deep_clone } )
-    {
-        #  Since most Cache::Cache's do their own deep cloning
-        #  we try a bit of a hack to try to bypass that.
-        $self->foreach_policy( 'storage',
-            sub
-            {
-                my ( $self, $policy, $storage ) = @_;
-
-                #  Only try this hack on things that subclass behaviour
-                #  we understand.
-                if( $storage->isa( 'Cache::BaseCache' ) )
-                {
-                    my ( $backend );
-
-                    if( $backend = $storage->_get_backend() )
-                    {
-                        $object->set_size( undef );
-                        $object->set_key( undef );
-
-                        $backend->store( $storage->get_namespace(),
-                            $key, $object );
-                        return;
-                    }
-                }
-
-                #  Ok, we couldn't figure out how to do our dirty hack...
-                $storage->set_object( $key, $object, $param );
-            } );
-    }
-    else
-    {
-        $self->foreach_driver( 'storage',  'set_object',
-            $key, $object, $param );
-    }
+    $self->foreach_driver( 'storage',  'set_object',
+        $key, $object, $param );
 
     $self->auto_purge( 'set' );
 }
@@ -257,26 +174,30 @@ sub get
     my ( $self, $key ) = @_;
     my ( $object );
 
-    my $storage_policies  = $self->{ policies }->{ storage };
-    my $validity_policies = $self->{ policies }->{ validity };
-    foreach my $storage_policy ( @{$storage_policies->{ order }} )
-    {
-        my $storage = $storage_policies->{ drivers }->{ $storage_policy };
-        next unless defined( $object = $storage->get_object( $key ) );
-
-        foreach my $validity_policy ( @{$validity_policies->{ order }} )
+    $self->foreach_policy( 'storage',
+        sub
         {
-            next if $validity_policies->{ drivers }->{ $validity_policy }->is_valid( $self, $storage, $object );
-            #  TODO: should remove from this storage. optionally?
-            undef $object;
-            last;
-        }
-        last if defined $object;
-    }
+            my ( $self, $policy, $storage ) = @_;
 
-    #  Check of auto_purge_on_get isn't strictly neccessary but
-    #  it saves the cost of a method call in the failure case.
-    $self->auto_purge( 'get' ) if $self->{ auto_purge_on_get };
+            $object = $storage->get_object( $key );
+            return unless defined $object;
+
+            $self->foreach_policy( 'validity',
+                sub
+                {
+                    my ( $self, $policy, $validity ) = @_;
+
+                    return if $validity->is_valid( $self, $storage, $object );
+
+                    undef $object;
+                    #  TODO: should remove from this storage. optionally?
+                    $self->last();
+                } );
+
+            $self->last() if defined $object;
+        } );
+
+    $self->auto_purge( 'get' );
 
     return( $object->get_data() ) if defined $object;
     return( undef );
@@ -306,11 +227,7 @@ sub set_object
     #  Backwards compat with Cache::Object objects.
     unless( $object->isa( 'Cache::CacheFactory::Object' ) )
     {
-        my ( $param );
-
-        $param = {};
-        $param->{ no_deep_clone } = 1 if $self->{ no_deep_clone };
-        $object = Cache::CacheFactory::Object->new_from_old( $object, $param );
+        $object = Cache::CacheFactory::Object->new_from_old( $object );
         #  TODO: compat with expires_at
     }
 
@@ -324,106 +241,32 @@ sub remove
     $self->foreach_driver( 'storage', 'remove', $key );
 }
 
-#
-#  CacheFactory extensions.
-sub exists
-{
-    my ( $self, $key ) = @_;
-    my ( $exists );
-
-    $self->foreach_policy( 'storage',
-        sub
-        {
-            my ( $self, $policy, $storage ) = @_;
-
-            #  If they've implemented an exists method, use it,
-            #  otherwise just do it the slow way.
-            if( $storage->can( 'exists' ) )
-            {
-                $exists = $storage->exists( $key );
-            }
-            else
-            {
-                $exists = defined( $storage->get_object( $key ) );
-            }
-
-            return $self->last() if $exists;
-        } );
-
-    return( $exists ? 1 : 0 );
-}
-
-
-#
-#  These following provide Cache::Memcached style interface.
-#    get_multi(), incr() and decr() cannot be "properly" implemented
-#    to use underlying functions because our object wrapper prevents
-#    the operations being single calls to the storage policy's
-#    implementation (if they have one), this then directly negates
-#    the purpose of these methods existing in the first place.
-sub delete
-{
-    my ( $self, $key ) = @_;
-
-    $self->remove( $key );
-}
-
-sub add
-{
-    my $self = shift;
-
-    $self->set( @_, mode => 'add' );
-}
-
-sub replace
-{
-    my $self = shift;
-
-    $self->set( @_, mode => 'replace' );
-}
-
-#  
-#sub get_multi
-#{
-#    my ( $self, @keys ) = @_;
-#}
-
-#sub incr
-#{
-#    my ( $self, $key, $value ) = @_;
-#}
-
-#sub decr
-#{
-#    my ( $self, $key, $value ) = @_;
-#}
-
 sub Clear
 {
-    my ( $self, @args ) = @_;
+    my ( $self ) = @_;
 
-    $self->foreach_driver( 'storage', 'Clear', @args );
+    $self->foreach_driver( 'storage', 'Clear' );
 }
 
 sub clear
 {
-    my ( $self, @args ) = @_;
+    my ( $self ) = @_;
 
-    $self->foreach_driver( 'storage', 'clear', @args );
+    $self->foreach_driver( 'storage', 'clear' );
 }
 
 sub Purge
 {
-    my ( $self, @args ) = @_;
+    my ( $self ) = @_;
 
-    $self->purge( @args );
+    $self->purge();
 }
 
 sub purge
 {
-    my ( $self, @args ) = @_;
+    my ( $self ) = @_;
 
-    $self->foreach_driver( 'pruning', 'purge', $self, @args );
+    $self->foreach_driver( 'pruning', 'purge', $self );
 }
 
 sub auto_purge
@@ -450,7 +293,7 @@ sub auto_purge
 
 sub Size
 {
-    my ( $self, @args ) = @_;
+    my ( $self ) = @_;
     my ( $size );
 
     $size = 0;
@@ -459,9 +302,7 @@ sub Size
         {
             my ( $self, $policy, $driver ) = @_;
 
-            #  Cache::FastMemoryCache 0.01 dies on Size(), workaround.
-            return if $driver->isa( 'Cache::FastMemoryCache' );
-            $size += $driver->Size( @args );
+            $size += $driver->Size();
         } );
 
     return( $size );
@@ -495,8 +336,6 @@ sub get_namespaces
         {
             my ( $self, $policy, $driver ) = @_;
 
-            #  Cache::NullCache->get_namespaces() dies, workaround it.
-            return $self->last() if $driver->isa( 'Cache::NullCache' );
             foreach my $namespace ( $driver->get_namespaces() )
             {
                 $namespaces{ $namespace }++;
@@ -533,8 +372,6 @@ sub get_identifiers
     return( $self->get_keys() );
 }
 
-
-
 sub set_positional_set
 {
     my ( $self, $positional_set ) = @_;
@@ -593,22 +430,6 @@ sub get_default_expires_in
     return( $time_pruning->get_default_expires_in() )  if $time_pruning;
 }
 
-sub limit_size
-{
-    my ( $self, $size ) = @_;
-    my ( $size_policy );
-
-    $size_policy = $self->get_policy_driver( 'pruning', 'size' );
-
-    unless( $size_policy )
-    {
-        carp "Cannot limit_size() when no 'size' pruning policy is set.";
-        return;
-    }
-
-    $size_policy->limit_size( $self, $size );
-}
-
 sub set_last_auto_purge
 {
     my ( $self, $last_auto_purge ) = @_;
@@ -835,35 +656,38 @@ sub get_policy_drivers
 #  and keeps me sane(ish).  Oh for a native ordered-hashref.
 sub last
 {
-#    my ( $self ) = @_;
-    $_[ 0 ]->{ _last } = 1;
+    my ( $self ) = @_;
+
+    $self->{ _last } = 1;
 }
 
 sub foreach_policy
 {
     my ( $self, $policytype, $closure ) = @_;
+    my ( $policies );
 
-    my $policies = $self->{ policies }->{ $policytype };
+    $policies = $self->{ policies }->{ $policytype };
     foreach my $policy ( @{$policies->{ order }} )
     {
         $closure->( $self, $policy, $policies->{ drivers }->{ $policy } );
         next unless $self->{ _last };
         delete $self->{ _last };
-        return;
+        last;
     }
 }
 
 sub foreach_driver
 {
     my ( $self, $policytype, $method, @args ) = @_;
+    my ( $policies );
 
-    my $policies = $self->{ policies }->{ $policytype };
+    $policies = $self->{ policies }->{ $policytype };
     foreach my $policy ( @{$policies->{ order }} )
     {
         $policies->{ drivers }->{ $policy }->$method( @args );
         next unless $self->{ _last };
         delete $self->{ _last };
-        return;
+        last;
     }
 }
 
@@ -952,7 +776,7 @@ __END__
 
 =head1 NAME
 
-Cache::CacheFactory - Factory class for Cache::Cache and other modules.
+Cache::CacheFactory - factory class for Cache::Cache and other modules.
 
 =head1 SYNOPSIS
 
@@ -1001,33 +825,10 @@ See L</"OPTIONS"> below for more details on possible options.
 
 =item $cache->set( key => $key, data => $data, [ expires_in => $expires_in, %additional_args ] )
 
-=item $cache->add( key => $key, data => $data, [ expires_in => $expires_in, %additional_args ] )
-
-=item $cache->replace( key => $key, data => $data, [ expires_in => $expires_in, %additional_args ] )
-
 =item $cache->set( $key, $data, [ $expires_in ] ) (only in compat-mode)
 
-=item $cache->add( $key, $data, [ $expires_in ] ) (only in compat-mode)
-
-=item $cache->replace( $key, $data, [ $expires_in ] ) (only in compat-mode)
-
 Associates C<$data> with C<$key> in the cache.
 
-C<< $cache->add() >> is a special form of C<< $cache->set() >> that will
-set the key if-and-only-if it doesn't already exist in the cache.
-
-C<< $cache->replace() >> is a special form of C<< $cache->set() >> that will
-set the key if-and-only-if it does already exist in the cache.
-
-B<Note>: the existence test and set in C<< $cache->add() >> and
-C<< $cache->replace() >> is B<NOT> an atomic operation, if you
-have a shared cache you will need to implement your own locking
-mechanism if you need to rely on this behaviour.
-
-A deep copy of C<$data> will automatically be taken if it is a reference,
-you can turn this behaviour off with the cache option C<no_deep_copy>
-detailed in L</"OPTIONS"> below.
-
 C<$expires_in> indicates the time in seconds until this data should be
 erased, or the constant C<$EXPIRES_NOW>, or the constant C<$EXPIRES_NEVER>.
 Defaults to C<$EXPIRES_NEVER>. This variable can also be in the extended
@@ -1086,19 +887,6 @@ that contains a fresh cached copy.
 Removes the data associated with C<$key> from each of the storage policies
 in this cache.
 
-=item $cache->delete( $key );
-
-This is a convenience alias for C<< $cache->remove( $key ) >>.
-
-=item $boolean = $cache->exists( $key );
-
-Returns true if data associated with C<$key> exists in the cache and
-false if there is no data associated with that key.
-
-This method makes no assumption about the form of the data stored:
-if you store a value of C<undef> you will still get a true return from
-C<< $cache->exists() >>.
-
 =item $object = $cache->get_object( $key );
 
 Returns the L<Cache::CacheFactory::Object> used to store the underlying
@@ -1155,7 +943,7 @@ is strongly deprecated.
 
 =item $cache->purge();
 
-Applies the pruning policy to all data in this namespace.
+Applies the pruning policy to all data in this namepace.
 
 =item $size = $cache->Size();
 
@@ -1208,13 +996,13 @@ Sets or gets the timestamp of the last auto-purge.
 See the documention for C<last_auto_purge> in L</"OPTIONS">
 for further details.
 
-=item $cache->set_auto_purge_on_set( 0 | 1 );
+=item $cache->set_auto_purge_on_set( 0 | 1 )
 
-=item $cache->set_auto_purge_on_get( 0 | 1 );
+=item $cache->set_auto_purge_on_get( 0 | 1 )
 
-=item $boolean = $cache->get_auto_purge_on_set();
+=item $boolean = $cache->get_auto_purge_on_set()
 
-=item $boolean = $cache->get_auto_purge_on_get();
+=item $boolean = $cache->get_auto_purge_on_get()
 
 Turns auto-purging on/off for C<< $cache->set() >> or
 C<< $cache->get() >>, or returns the current state
@@ -1241,18 +1029,9 @@ C<auto_purge_on_get_interval> options.
 
 Look at L</"OPTIONS"> for further details.
 
-=item $cache->limit_size( $size );
-
-Only available if a pruning policy of 'size' has been set,
-this method will allow you to perform a one-off prune of
-the storage policies to C<$size> size or below.
-
-This behaves like the C<limit_size()> method of
-L<Cache::SizeAwareCache>.
-
 =back
 
-=head1 NON-OBJECT-ORIENTATED FUNCTIONS
+=head1 non-OO functions
 
 =over
 
@@ -1286,24 +1065,6 @@ supply C<:best_available> on the use line to import them.
 
 =back
 
-=head1 CONSTANTS
-
-You can export the following constants:
-
-=over
-
-=item $NO_MAX_SIZE
-
-You can export this with C<< use Cache::CacheFactory qw/$NO_MAX_SIZE/; >>
-and supply it to the C<max_size> option of a 'size' pruning policy.
-
-This value of C<$NO_MAX_SIZE> is compatible with that defined by
-L<Cache::SizeAwareCache>, so you can use either source.
-
-See L<Cache::CacheFactory::Expiry::Size> for further details.
-
-=back
-
 =head1 OPTIONS
 
 The following options may be passed to the C<new()> constructor:
@@ -1483,32 +1244,6 @@ is missing and will instead generate a C<warn>.
 If you also set C<nonwarning_missing_policies> to a true value,
 this C<warn> will also be surpressed.
 
-=item no_deep_clone => 0 | 1
-
-Setting C<no_deep_clone> to a true value will prevent the
-default behaviour of taking a deep clone of the data provided to
-C<< $cache->set() >>.
-
-This can be a performance gain if you don't need to be paranoid
-about the cache sharing references with whatever handed them in,
-or if you want to handle the cloning yourself within your application.
-
-Regretfully C<no_deep_clone> on the cache can only act in an
-advisory capacity to storage policies, they may choose to
-disregard the flag and many of the L<Cache::Cache> modules
-will do just this. (Not unreasonably: they predate
-L<Cache::CacheFactory> considerably.) L<Cache::CacheFactory>
-tries hard to convince them to avoid taking clones but may or
-may not succeed depending on precisely what you're attempting,
-you'll have to suck it and see I'm afraid.
-
-Using this option with a storage policy of 'memory' will provide
-you with similar behaviour to L<Cache::FastMemoryCache>, with
-the exception that, unavoidably, a deep clone is always created
-on C<< $cache->get() >>. If this is undesirable, install
-L<Cache::FastMemoryCache> and use a storage policy of 'fastmemory'
-in conjunction with setting C<no_deep_clone>.
-
 =back
 
 =head1 POLICIES
@@ -1546,13 +1281,6 @@ per-process in-memory caching.
 Implemented using L<Cache::SharedMemoryCache>,
 this provides in-memory caching with the cache shared between processes.
 
-=item fastmemory
-
-Implemented using L<Cache::FastMemoryCache>,
-this provides in-memory caching like the 'memory' policy but with
-all the deep-copies of data stripped out, best used in conjunction
-with the C<no_deep_clone> option set on the cache.
-
 =item null
 
 Implemented using L<Cache::NullCache>, this cache is
@@ -1565,7 +1293,7 @@ used to provide a fake cache that never stores anything.
 All I<pruning> and I<validity> policies are interchangable, the difference
 between the two is when the policy is applied:
 
-A pruning policy is applied when you C<purge()> or periodically if
+An pruning policy is applied when you C<purge()> or periodically if
 C<auto_purge_on_set> or C<autopurge_on_get> is set, it removes all
 entries that fail the policy from the cache. Note that an item can
 be I<eligible> to be pruned but still be in the cache and fetched
@@ -1798,47 +1526,11 @@ version if they prove useful and won't over-complicate matters - for
 now it's best to create a wrapper module that internally creates the
 caches seperately but presents the Cache::Cache API externally.
 
-=item Add/replace aren't atomic
-
-The C<< $cache->add/replace() >> methods aren't atomic, this mostly
-defeats their purpose in a shared-cache situation. This could be
-considered a bug.
-
-=item Aren't there a million Cache::Cache replacements already?
-
-At the time I started writing L<Cache::CacheFactory> I'd been trying
-to find a caching solution that had the combination of features I
-needed, I had no luck in finding one.
-
-Since then I've found a couple of other similar modules, and more
-have been written and released, you may or may not find them suiting
-your needs more closely, so I suggest taking a good look:
-
-L<CHI> - this module appears to have much the same motivation and
-strategy as L<Cache::CacheFactory> in terms of storage policies,
-however, from what I can gather, it doesn't appear to split
-validity/pruning policies into seperate and/or combinable modules.
-
-L<Cache> - not sure how I missed this one when I was researching,
-it's a mature module that gives you flexibile validity/pruning
-policies but doesn't have such a wide range of storage policies
-available.
-
-Please note that these descriptions are from my own imperfect
-understanding of the modules concerned, by no means take them
-as an authorative description of their functionality. Please
-feel free to contact me if I've included any inaccuracies. :)
-
 =back
 
 =head1 SEE ALSO
 
-L<Cache::Cache>, L<Cache::CacheFactory::Object>,
-L<Cache::CacheFactory::Expiry::Base>,
-L<Cache::CacheFactory::Expiry::Time>,
-L<Cache::CacheFactory::Expiry::Size>,
-L<Cache::CacheFactory::Expiry::LastModified>,
-L<Cache::FastMemoryCache>
+L<Cache::Cache>, L<Cache::CacheFactory::Object>
 
 =head1 SUPPORT
 
@@ -1883,12 +1575,9 @@ work is done by this module and its subclasses.
 Chris Winters for L<Class::Factory>, saving me the trouble of finding
 out what policy modules are or aren't available.
 
-John Millaway for L<Cache::FastMemoryCache>, which inspired the
-C<no_deep_clone> option.
-
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2008-2010 Sam Graham, all rights reserved.
+Copyright 2008 Sam Graham, all rights reserved.
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
