@@ -2,7 +2,7 @@
 # Purpose : Cache Storage Policy Factory.
 # Author  : Sam Graham
 # Created : 23 Jun 2008
-# CVS     : $Id: Storage.pm,v 1.8 2010-02-09 12:30:58 illusori Exp $
+# CVS     : $Id: Storage.pm,v 1.3 2008-06-27 11:58:10 illusori Exp $
 ###############################################################################
 
 package Cache::CacheFactory::Storage;
@@ -14,7 +14,8 @@ use Class::Factory;
 
 use base qw/Class::Factory/;
 
-$Cache::CacheFactory::Storage::VERSION = '1.09_01';
+$Cache::CacheFactory::Storage::VERSION =
+    sprintf"%d.%03d", q$Revision: 1.3 $ =~ /: (\d+)\.(\d+)/;
 
 sub new
 {
@@ -34,8 +35,6 @@ __PACKAGE__->register_factory_type(
     file         => 'Cache::FileCache' );
 __PACKAGE__->register_factory_type(
     null         => 'Cache::NullCache' );
-__PACKAGE__->register_factory_type(
-    fastmemory   => 'Cache::FastMemoryCache' );
 
 1;
 
@@ -45,7 +44,7 @@ __END__
 
 =head1 NAME
 
-Cache::CacheFactory::Storage - Factory class for storage policies for Cache::CacheFactory.
+Cache::CacheFactory::Storage - factory class for storage policies for Cache::CacheFactory.
 
 =head1 DESCRIPTION
 
@@ -79,7 +78,7 @@ Last author:     $Author: illusori $
 
 =head1 COPYRIGHT
 
-Copyright 2008-2010 Sam Graham.
+Copyright 2008 Sam Graham.
 
 This library is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
